# -*- coding: utf-8 -*-

"""
DB API felületen különböző műveleteket végez 
a linkgroup adatbázissal 

"""

def clearDb(db):
   """
   Legyalul minden adatot az adatbázisból, hogy
   tiszta lappal kezdhessünk.

   @type    db: db.Connection
   @param   db: Adatbázis kapcsolat

   """

   # Kérünk kurzort
   cur = db.cursor()

   # Ez a csatolás (cascade) miatt törli 
   # a kapcsolatokat is
   cur.execute("""
      DELETE FROM people
   """)

   # Érvényesítjük a változtatásoakt
   db.commit()

def loadFromFile(fn, db):
   """
   Betölti a bemeneti gráf éleit a bemeneti fájlból
   és behelyezi az adatokat az adatbázisba.
   A fájl egy-egy sora az él végpontjainak neveit
   tartalmazza.

   @type    fn: string
   @param   fn: A bemeneti fájl neve
   @type    db: db.Connection
   @param   db: Adatbázis kapcsolat

   """

   # A pontnév-párok listája 
   pairs = map(
      lambda x: x.split(),
      filter(
         lambda l: l, 
         map(
            lambda l: l.strip(), 
            file(fn).readlines()
         )
      )
   )

   # Először behelyezzük az embereket a db-be.
   # Ehhez szükségünk van a nevek listájára

   # A halmaz mindenkit csak egyszer fog tartalmazni
   people = set()
   for a, b in pairs:
      people.add(a)
      people.add(b)

   # Halmazról listára konvertáljuk
   people = list(people)

   # Kérünk egy kurzort
   cur = db.cursor()

   cur.executemany("""
      INSERT INTO people (name) VALUES (%s)
      """,
      people
   )

   # Ezután behelyezzük a kapcsolatokat
   cur.executemany("""
      INSERT INTO connections (author_1, author_2)
      SELECT p1.id, p2.id
      FROM people p1, people p2
      WHERE p1.name = %s
      AND p2.name = %s
      """,
      pairs
   )

   # Érvényesítjük a változtatásokat
   db.commit()

def calcDegree(db):
   """
   Kiszámolja és frissíti az adatbázisban az egyes
   emberek kapcsolatainak számát (fokszámát).

   @type    db: db.Connection
   @param   db: Adatbázis kapcsolat

   """
   
   cur = db.cursor()

   cur.execute("""
      UPDATE
         people p
         JOIN (
            SELECT id, COUNT(*) deg
            FROM (
               SELECT author_1 AS id FROM connections
               UNION ALL
               SELECT author_2 AS id FROM connections
            ) t
            GROUP BY id
         ) degTable
         ON
            p.id = degTable.id
      SET
         p.deg = degTable.deg
   """)

   # Ne felejtsük érvényesíteni a változtatásokat
   db.commit()

# Konstansok L{getDegrees} által visszaadott párok indexeléséhez
IDD_ID = 0
IDD_DEG = 1

def getDegrees(db):
   """
   Visszaadja az embereket és fokszámaikat fokszám szerint csökkenő
   sorrendben rendezve.

   Feltételezi, hogy a fokszámok már ki vannak számolva
   (ehhez meg kellet hívni a L{calcDegree} függvényt).

   @type    db: db.Connection
   @param   db: Adatbázis kapcsolat

   @rtype:  (azonosító, fokszám) lista
   @return: Az emberek és fokszámaik.

   """
 
   cur = db.cursor()

   cur.execute("""
      SELECT id, deg FROM people ORDER BY deg DESC
   """)

   # Commit most nem kell, mivel nem változtattunk,
   # csak lekérdeztünk

   return cur.fetchall()

def deleteUnderDeg(db, minDeg):
   """
   Kitörli az adatbázisból azokat az embereket, akiknek
   a fokszáma kisebb, mint minDeg.
   
   Feltételezi, hogy a fokszámok már ki vannak számolva
   (ehhez meg kellet hívni a L{calcDegree} függvényt).
   A függvény meghívása után az adatbázisban érvénytelen fokszámok
   fognak szerepelni, ezért érdemes lehet utána újból
   meghívni a L{calcDegree} függvényt.

   @type    db: db.Connection
   @param   db: Adatbázis kapcsola
   @type    minDeg: number
   @param   minDeg: Túléléshez szükséges minimális fokszám

   """

   cur = db.cursor()

   cur.execute("""
      DELETE FROM people WHERE deg <= %s
      """,
      minDeg
   )

   # Mehet
   db.commit()
